#!/usr/bin/perl
#
#--------------------------------------------------------------------------
# flooble.com pad : a web based notepad script (version 0.1)
# Copyright (C) 2001 Lev Epshteyn (http://www.levik.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#-------------------------------------------------------------------------
#
# This is one of the first perl scripts i have written. I apologize
# for the quality of the code, and have tried to make up for it though
# thorough documentation. 
# 
# CGI.pm is required (see http://stein.cshl.org/WWW/software/CGI/)
#
# Please see the README file for more information and contact me with 
# any questions/comments.
#      - Lev Epshteyn (http://www.levik.com)
#
use CGI;

use strict vars;

#
# Initializing some configuration params. Change these to reflect your setup:
############################

# The url of the script
my $self="http://www.flooble.com/cgi-bin/pad.pl";

# Directory containing the pad data files (make sure it's writable by the script)
my $data_dir = "paddata/";

# Maximum length of the pad contents (in characters)
my $max_length = 2048;

# The value of the "secure" cookie to indicate that a secure pad is selected
my $cookie_value = 'true';

#
# Initializing runtime variables:
############################

# Used as a boolean to see if the logon form shold be shown in place of the secure
# pad
my $logon = "";

# Initialize the query object (used to access URL params, cookies, http headers)
my $query = new CGI;

# Get the URL of the referring page (so we know where to reload to)
my $referer = $query->referer();

# Get the 'action' url parameter this controls what the script does.
my $action = $query->param("action"); 

# Text token
my $text = "";

# The 'js' url parameter is set into a variable. If it exists, the script will behave
# as a javascript include. Otherwise it will display straight HTML code
my $js = $query->param('js');

# The 'id' url parameter indicates the id of the pad desired. 
my $id = $query->param('id');

# The name of the security cookie for this pad. (Includes $id to be independent of
# other pads)
my $cookie_name = 'pad'.$id.'secure';

# The name of the password cookie for this pad. (Includes $id as well. Password is stored
# in plain text, may be fixed in future versions.)
my $password_name = 'pad'.$id.'password';

# Stores the filename of the file containing the pad text
my $file;

# Hash to store the pad owner's properties
my %properties;

#
# Sets up the "refresh" code. This is the javascript code executed by the popup window 
# to refresh the page in the main window after one of the cookies was changed. There are two 
# ways of doing this:
#   1. By setting "opener.location" to the referrer URL
#   2. By calling "opener.location.reload()"
# However the second method is blocked by the security engines of the newer browsers, so it's
# only used if for some reason the referrer url is unavailable. (If both these methods fail, 
# the user will have to reload the page manually to see the change in the pad.)
############################
my $refresh_code;
if ($referer) {
  $refresh_code = "opener.location='$referer';"
} else {
  $refresh_code = "opener.location.reload(true);"
}

#
# If no action is specified, we just want to show the pad's contents (or for a secure pad,
# possibly the logon form).
############################
if (!$action) {

  # Default content type is 'text/html' ...
  my $type='text/html';

  # ... unless this is a JavaScript input.
  if ($js) { $type='application/x-javascript'; }

  # Print the header. Expires is set to 'now' so that the script is (hopefully) reloaded
  # each time. (This doesn't seem to work with the Opera browser)
  print $query->header(-type=>$type,
                       -expires=>'now');

  # Load the properties hash for $id.
  %properties = getConfig($id);

  # If the 'secure' cookie ($cookie_name) is not set to $cookie_value use the plain pad file,
  # otherwise use the secure file (In this case, also check if the password is properly set in
  # a cookie specified by $password_name. If not, set the logon flag to 'true'.)
  if ($query->cookie($cookie_name) ne $cookie_value) {
    $file = $data_dir.$properties{'plain'}; 
  } else {
    $file = $data_dir.$properties{'secure'}; 
    if ($query->cookie($password_name) ne $properties{'password'}) {
       $logon="true";
    }
  }

  # Output the form header.
  &writeln('');
  &write ("<form action=\"$self\" method=\"post\" target=\"pad_submit\">");
  &write("<table border=\"0\"><tr><td align=\"left\"> ");
  &write("<a href=\"http://www.flooble.com/\"><img src=\"http://www.flooble.com/images/blobicon.gif\" width=37 height=26 border=\"0\" align=\"left\"></a>");
  &write("</td><td valign=\"bottom\" align=\"right\">");
  &writeln("<b>$properties{'name'}</b>'s pad:");

  # Show the 'plain | secure' tabs (one as link, one as text) depending on the value of the secure cookie.
  if ($query->cookie($cookie_name) eq $cookie_value) {
    &write ("|<a href=\"$self?action=setplain&id=$id\" target=\"pad_submit\" onclick=\"win=window.open('about:blank','pad_submit', 'toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbar=no,resizable=no,copyhistory=no,width=30,height=30');\">plain</a>| ");
    &write ("|secure|<br> ");
  } else {
    &write ("|plain| ");
    &write ("|<a href=\"$self?action=setsecure&id=$id\" target=\"pad_submit\" onclick=\"win=window.open('about:blank','pad_submit', 'toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbar=no,resizable=no,copyhistory=no,width=30,height=30');\">secure</a>|<br>");
  }
  &write("</td></tr></table>");
  &write ("<input type=\"hidden\" name=\"id\" value=\"$id\">");
  
  my $label;  

  # If the $logon flag is not set, show the contents of the selected pad:
  if (!$logon) {

    # Open the proper file and read its content in. Then close it.
    open(IN,"<$file");
    my @lines=<IN>;
    close(IN);

    &write ("<input type=\"hidden\" name=\"action\" value=\"save\">");
    
    # Output the textarea tag with the rows/columns from the user's properties.
    &write ("<textarea name=\"text\" wrap=\"hard\" rows=$properties{'rows'} cols=$properties{'cols'}>");

    # Take each line, remove CR/LF characters and output them.
    foreach $text (@lines) {
      $text =~ s/\r//g;
      $text =~ s/\n//g;
      &writeln($text);
    }

    # Close textarea. Output flooble link.
    &write("</textarea><br>");
    &write("<small><a href=\"http://www.flooble.com/pad/\">Get your free pad</a></small> ");
    
    # If showing a secure pad, show the logout link.
    if ($query->cookie($cookie_name) eq $cookie_value) {
      &write(" [<a href=\"$self?action=logout&id=$id\" target=\"pad_submit\" onclick=\"win=window.open('about:blank','pad_submit','toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbar=no,resizable=no,copyhistory=no,width=30,height=30');\">logout</a>] ");
    }

    # Output the 'clear' and 'reset' buttons. Set the main submit button label to 'save'.
    &write("<input type=button value=clear onclick=\"this.form.text.value=''\"> ");
    &write("<input type=reset value=reset> ");
    $label = 'save';

  # If the $logon flag is set, show the logon form for the secure pad and set the main submit
  # button label to 'login'.
  } else {
    &write ("<input type=\"hidden\" name=\"action\" value=\"login\">");
    &write ("<hr width=\"100%\" size=\"1\"> Please enter your password: <br>");
    &write ("<input type=\"checkbox\" name=\"remember\"> ");
    &write ("<input type=\"password\" name=\"password\" size=\"12\"> ");
    $label = 'login';
  }

  # Show the main submit button now with the label specified above (either 'save' or 'login')
  &write ("<input type=\"button\" value=\"$label\" onclick=\"win=window.open('about:blank','pad_submit', 'toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbar=no,resizable=no,copyhistory=no,width=30,height=30');this.form.submit();\">"); 

  # If the logon flag is set, the flooble link has not been shown yet. Show it now.
  if ($logon) {
    &write("<br><small><a href=\"http://www.flooble.com/pad/\">Get your free pad</a></small> ");
  }

  # Close the form.
  &write ("</form>");

#
# If action is 'save' the text data must be stored in the proper file, and the main 
# window refreshed.
############################

} elsif ($action eq 'save') {

  # Load the properties
  %properties = getConfig($id);

  # Select the plain or the secure file to use. Check password.
  if ($query->cookie($cookie_name) ne $cookie_value) {
    $file = $data_dir.$properties{'plain'};
  } elsif ($query->cookie($password_name) eq $properties{'password'}) {
    $file = $data_dir.$properties{'secure'};
  }  

  # Get the text from the CGI parameter, then grab the first $max_length characters.
  $text = $query->param("text");
  if (length($text) > $max_length) {
    $text = substr($text,0,$max_length);
  }

  # Make sure noone's tryin any funny business by putting a closed textarea tag into the
  # text. (This would allow the display of arbitrary code on the pad page.)
  $text =~ s/<\/([tT][eE][xX][tT][aA][rR][eE][aA])/< \/$1/g;
  
  # Save the text to the proper file.
  open(OUT,">$file");
  print(OUT $text);
  close(OUT);

  # Display a small HTML document to close the popup window
  # (where this document should be shown)
  print($query->header);
  print("<head><title>saving</title></head><body onload=\"this.window.close()\">");
  print("Saving...</body>");

#
# If the action is 'setsecure' we wish to set the secure cooke ($cookie_name) to the value
# specified at the top of the program. This will let pad know that it should show the
# secure version.
############################

} elsif ($action eq 'setsecure') {

  # Set the cookie to last a month.
  my $cookie = $query->cookie(-name=>$cookie_name,-value=>$cookie_value,-expires=>'+30d');

  # Display the opener reloading, self closing document.
  print($query->header(-expires=>'now',-cookie=>$cookie,-type=>'text/html'));
  print("<head><title>switching</title></head><body onload=\"".$refresh_code."this.window.close()\">");
  print("Switching...</body>");

#
# If the action is 'login' we need to set the cookie containing the password entered by the 
# user. (Since password checking is done on every execution, there is no need to check that
# the password be correct at this point.)
############################

} elsif ($action eq 'login') {
  my $cookie;

  # If the 'remember' checkbox is checked, cookie will last 30 days. Otherwise it will die when the
  # browser is closed.
  if ($query->param('remember')) {
    $cookie = $query->cookie(-name=>$password_name,-value=>$query->param(password),-expires=>'+30d'); 
  } else { 
    $cookie = $query->cookie(-name=>$password_name,-value=>$query->param(password)); 
  }

  # Display the opener reloading, self closing document.
  print($query->header(-expires=>'now',-cookie=>$cookie,-type=>'text/html'));
  print("<head><title>login</title></head><body onload=\"".$refresh_code."this.window.close()\">");
  print("Logging in...</body>");

#
# If the action is 'logout', clear the password cookie 
############################

} elsif ($action eq 'logout') {
  my $cookie = $query->cookie(-name=>$password_name,-value=>'');

  # Show document
  print($query->header(-expires=>'now',-cookie=>$cookie,-type=>'text/html'));
  print("<head><title>logout</title></head><body onload=\"".$refresh_code."this.window.close()\">");
  print("Logging out...</body>");

#
# If the action is 'setplain', clear the secure cookie ($cookie_name), so that
# the pad will now show the plain version.
############################

} elsif ($action eq 'setplain') {
  my $cookie = $query->cookie(-name=>$cookie_name,-value=>'');

  # Show document
  print($query->header(-expires=>'now',-cookie=>$cookie,-type=>'text/html'));
  print("<head><title>switching</title></head><body onload=\"".$refresh_code."this.window.close()\">");
  print("Switching...</body>");

#
# The action 'switch' toggles between the plain and secure states, depending on which state the
# pad is currently in (as determined by the secure cookie). This is not currently used 
# (I use 'setplain' and 'setsecure' instead, but it will probably be used in an upcoming version)
############################
 
} elsif ($action eq 'switch') {
  my $cookie;
  if ($query->cookie($cookie_name) ne $cookie_value) {
    $cookie = $query->cookie(-name=>$cookie_name,-value=>$cookie_value,-expires=>'+30d');
  } else {
    $cookie = $query->cookie(-name=>$cookie_name,-value=>'');
  }

  # Show document
  print("<head><title>switching</title></head><body onload=\"".$refresh_code."this.window.close()\">");
  print("Switching...</body>");
}

#
# The writeX subroutines allow the script the flexibility of being both a JavaScript include
# and a regular CGI. By checking the value of the global $js flag, they determine how the output
# should be formatted.
############################

# Writes javascript code. If in js modes, simply prints it, otherwise surrounds with a script tag.
sub write_js {
  my ($param) = @_;
  if ($js) {
    print($param);
  } else {
    print "<SCRIPT language=\"javascript\">$param</SCRIPT>";
  }
}

# Writes text. If in js mode, escapes double quotes and backslashes then uses "document.write()",
# otherwise simply calls print.
sub write {
  my ($param) = @_;
  if ($js) {
    $param =~ s/(["\\])/\\$1/g;
    print ("document.write(\"$param\");\n");
  } else {
    print ($param);
  }
}
 
# Same as write() but with a newline character at the end.
sub writeln {
  my ($param) = @_;
  if ($js) {
    $param =~ s/(["\\])/\\$1/g;
    print ("document.writeln(\"$param\");\n");
  } else {
    print "$param \n";
  }
} 

#
# Loads the config file for a given id. If the id is not specified or the
# specified id has no config the script dies with an error. Returns a hash
# of all properties.
############################

sub getConfig {
  my ($param) = @_;
  if (!$param) { 
    &write("no id"); 
    exit 0;
  }
  my %properties; 
  if (!open (IN,"<$data_dir$param.pad")) {
    &write("bad id: $param");
    exit 0;
  }
  while (<IN>) {
    my ($key, $value) = split(/\s*=\s*/,$_,2);
    chomp($value);
    $properties{$key} = $value;  
  }  
  return %properties;
}
